@@ -39,8 +39,13 @@
 #  include <libintl.h>
 #endif
 
-static const char *glade_xml_tag = "GladeXML::";
-static const char *glade_xml_name_tag = "GladeXML::name";
+static const gchar *glade_xml_tree_key     = "GladeXML::tree";
+static GQuark       glade_xml_tree_id      = 0;
+static const gchar *glade_xml_name_key     = "GladeXML::name";
+static GQuark       glade_xml_name_id      = 0;
+static const gchar *glade_xml_tooltips_key = "GladeXML::tooltips";
+static GQuark       glade_xml_tooltips_id  = 0;
+
 
 static void glade_xml_init(GladeXML *xml);
 static void glade_xml_class_init(GladeXMLClass *class);
@@ -89,6 +94,10 @@ glade_xml_class_init (GladeXMLClass *class)
     parent_class = g_type_class_peek_parent (class);
 
     G_OBJECT_CLASS(class)->finalize = glade_xml_finalize;
+
+    glade_xml_tree_id = g_quark_from_static_string(glade_xml_tree_key);
+    glade_xml_name_id = g_quark_from_static_string(glade_xml_name_key);
+    glade_xml_tooltips_id = g_quark_from_static_string(glade_xml_tooltips_key);
 }
 
 static void
@@ -96,7 +105,7 @@ glade_xml_init (GladeXML *self)
 {
     GladeXMLPrivate *priv;
 	
-    self->priv = priv = g_new (GladeXMLPrivate, 1);
+    self->priv = priv = g_new0 (GladeXMLPrivate, 1);
 
     self->filename = NULL;
 
@@ -108,9 +117,9 @@ glade_xml_init (GladeXML *self)
     priv->radio_groups = g_hash_table_new (g_str_hash, g_str_equal);
     priv->toplevel = NULL;
     priv->accel_groups = NULL;
-    priv->focus_ulines = NULL;
     priv->default_widget = NULL;
     priv->focus_widget = NULL;
+    priv->deferred_props = NULL;
 }
 
 /**
@@ -140,36 +149,6 @@ glade_xml_new(const char *fname, const char *root, const char *domain)
     return self;
 }
 
-/**
- * glade_xml_new_with_domain:
- * @fname: the XML file name.
- * @root: the widget node in @fname to start building from (or %NULL)
- * @domain: the translation domain to use for this interface (or %NULL)
- *
- * Creates a new GladeXML object (and the corresponding widgets) from the
- * XML file @fname.  Optionally it will only build the interface from the
- * widget node @root (if it is not %NULL).  This feature is useful if you
- * only want to build say a toolbar or menu from the XML file, but not the
- * window it is embedded in.  Note also that the XML parse tree is cached
- * to speed up creating another GladeXML object for the same file.  This
- * function differs from glade_xml_new in that you can specify a different
- * translation domain from the default to be used.
- *
- * Returns: the newly created GladeXML object, or NULL on failure.
- */
-GladeXML *
-glade_xml_new_with_domain(const char *fname, const char *root,
-			  const char *domain)
-{
-    GladeXML *self = g_object_new(GLADE_TYPE_XML, NULL);
-
-    if (!glade_xml_construct(self, fname, root, domain)) {
-	g_object_unref(G_OBJECT(self));
-	return NULL;
-    }
-    return self;
-}
-
 /**
  * glade_xml_construct:
  * @self: the GladeXML object
@@ -615,8 +594,8 @@ glade_get_widget_name(GtkWidget *widget)
 {
     g_return_val_if_fail(widget != NULL, NULL);
 
-    return (const char *)gtk_object_get_data(GTK_OBJECT(widget),
-					     glade_xml_name_tag);
+    return (const char *)g_object_get_qdata(G_OBJECT(widget),
+					    glade_xml_name_id);
 }
 
 /**
@@ -633,7 +612,7 @@ glade_get_widget_tree(GtkWidget *widget)
 {
     g_return_val_if_fail(widget != NULL, NULL);
 
-    return gtk_object_get_data(GTK_OBJECT(widget), glade_xml_tag);
+    return g_object_get_qdata(G_OBJECT(widget), glade_xml_tree_id);
 }
 
 /* ------------------------------------------- */
@@ -651,8 +630,6 @@ glade_get_widget_tree(GtkWidget *widget)
 void
 glade_xml_set_toplevel(GladeXML *xml, GtkWindow *window)
 {
-    static char *tooltips_key = "libglade::GladeXML::tooltips";
-
     if (xml->priv->focus_widget)
 	gtk_widget_grab_focus(xml->priv->focus_widget);
     if (xml->priv->default_widget)
@@ -667,9 +644,49 @@ glade_xml_set_toplevel(GladeXML *xml, GtkWindow *window)
     xml->priv->accel_groups = NULL;
     /* the window should hold a reference to the tooltips object */
     gtk_object_ref(GTK_OBJECT(xml->priv->tooltips));
-    gtk_object_set_data_full(GTK_OBJECT(window), tooltips_key,
-			     xml->priv->tooltips,
-			     (GtkDestroyNotify)gtk_object_unref);
+    g_object_set_qdata_full(G_OBJECT(window), glade_xml_tooltips_id,
+			    xml->priv->tooltips,
+			    (GDestroyNotify)gtk_object_unref);
+}
+
+/**
+ * glade_xml_handle_widget_prop
+ * @xml: the GladeXML object
+ * @widget: the property the widget to set the property on.
+ * @prop_name: the name of the property.
+ * @value_name: the name of the widget used as the value for the property.
+ *
+ * Some widgets have properties of type GtkWidget.  These are
+ * represented as the widget name in the glade file.  When
+ * constructing the interface, the widget specified as the value for a
+ * property may not exist yet.
+ *
+ * Rather than setting the property directly, this function should be
+ * used.  It will perform the name to GtkWidget conversion, and if the
+ * widget is yet to be constructed, defer setting the property until
+ * the widget is constructed.
+ */
+void
+glade_xml_handle_widget_prop(GladeXML *self, GtkWidget *widget,
+			     const gchar *prop_name, const gchar *value_name)
+{
+    GtkWidget *value_widget;
+
+    g_return_if_fail(GLADE_IS_XML(self));
+
+    value_widget = g_hash_table_lookup(self->priv->name_hash, value_name);
+    if (value_widget) {
+	g_object_set(G_OBJECT(widget), prop_name, value_widget, NULL);
+    } else {
+	GladeDeferredProperty *dprop = g_new(GladeDeferredProperty, 1);
+
+	dprop->target = G_OBJECT(widget);
+	dprop->prop_name = prop_name;
+	dprop->prop_value = value_name;
+
+	self->priv->deferred_props = g_list_prepend(self->priv->deferred_props,
+						    dprop);
+    }
 }
 
 /**
@@ -733,64 +750,6 @@ glade_xml_ensure_accel(GladeXML *xml)
     return (GtkAccelGroup *)xml->priv->accel_groups->data;
 }
 
-#if 0
-/**
- * glade_xml_handle_label_accel:
- * @xml: the GladeXML object.
- * @target: the target widget name (or %NULL).
- * @key: the key code for the accelerator.
- *
- * This function is called by the GtkLabel creation routine to register an
- * underline accelerator for the label.  If the target widget exists, the
- * accelerator is connected to the grab_focus signal.  If it does not exist
- * yet, the (target, key) pair is saved so that it can be attached when the
- * widget is created.
- * If @target is %NULL, then the accelerator should be attached to one of
- * the parents of this widget (typically a GtkButton).
- */
-void
-glade_xml_handle_label_accel(GladeXML *xml, const gchar *target, guint key)
-{
-	if (target) {
-		GtkWidget *twidget = glade_xml_get_widget(xml, target);
-
-		if (twidget)
-			gtk_widget_add_accelerator(twidget, "grab_focus",
-						   glade_xml_ensure_accel(xml),
-						   key, GDK_MOD1_MASK, 0);
-		else {
-			GladeFocusULine *uline = g_new(GladeFocusULine, 1);
-
-			uline->widget_name = target;
-			uline->key = key;
-			xml->priv->focus_ulines =
-				g_list_prepend(xml->priv->focus_ulines, uline);
-		}
-	} else
-		xml->priv->parent_accel = key;
-}
-
-/**
- * glade_xml_get_parent_accel:
- * @xml: the GladeXML object.
- *
- * This function gets the latest label underline accelerator directed at the
- * parent widget.  If there is no accelerator waiting, 0 is returned.  If
- * this function is called twice in a row, the second call will always return
- * 0.
- *
- * Returns: the key code for the accelerator, or zero.
- */
-guint
-glade_xml_get_parent_accel(GladeXML *xml)
-{
-	guint key = xml->priv->parent_accel;
-
-	xml->priv->parent_accel = 0;
-	return key;
-}
-#endif
-
 /* this is a private function */
 static void
 glade_xml_add_signals(GladeXML *xml, GtkWidget *w, GladeWidgetInfo *info)
@@ -854,8 +813,8 @@ remove_data_func(gpointer key, gpointer value, gpointer user_data)
     GObject *object = value;
     GladeXML *xml = user_data;
 
-    g_object_set_data(object, glade_xml_tag, NULL);
-    g_object_set_data(object, glade_xml_name_tag, NULL);
+    g_object_set_qdata(object, glade_xml_tree_id, NULL);
+    g_object_set_qdata(object, glade_xml_name_id, NULL);
 }
 
 static void
@@ -1116,6 +1075,7 @@ glade_register_widgets(const GladeWidgetBuildData *widgets)
 
 /**
  * glade_xml_set_value_from_string
+ * @xml: the GladeXML object.
  * @pspec: the GParamSpec for the property
  * @string: the string representation of the value.
  * @value: the GValue to store the result in.
@@ -1132,7 +1092,8 @@ glade_register_widgets(const GladeWidgetBuildData *widgets)
  * Returns: %TRUE on success.
  */
 gboolean
-glade_xml_set_value_from_string (GParamSpec *pspec,
+glade_xml_set_value_from_string (GladeXML *xml,
+				 GParamSpec *pspec,
 				 const gchar *string,
 				 GValue *value)
 {
@@ -1208,6 +1169,22 @@ glade_xml_set_value_from_string (GParamSpec *pspec,
 
 	    g_value_set_object(value, adj);
 	    gtk_object_sink(GTK_OBJECT(adj));
+	} else if (G_VALUE_HOLDS(value, GDK_TYPE_PIXBUF)) {
+	    gchar *filename;
+	    GError *error = NULL;
+	    GdkPixbuf *pixbuf;
+
+	    filename = glade_xml_relative_file(xml, string);
+	    pixbuf = gdk_pixbuf_new_from_file(filename, &error);
+	    if (pixbuf) {
+		g_value_set_object(value, pixbuf);
+		g_object_unref(G_OBJECT(pixbuf));
+	    } else {
+		g_warning("Error loading image: %s", error->message);
+		g_error_free(error);
+		ret = FALSE;
+	    }
+	    g_free(filename);
 	} else
 	    ret = FALSE;
 	break;
@@ -1244,6 +1221,7 @@ glade_standard_build_widget(GladeXML *xml, GType widget_type,
     static GArray *props_array = NULL;
     GObjectClass *oclass;
     GtkWidget *widget;
+    GList *deferred_props = NULL, *tmp;
     guint i;
 
     if (!props_array)
@@ -1264,7 +1242,22 @@ glade_standard_build_widget(GladeXML *xml, GType widget_type,
 	    continue;
 	}
 
-	if (glade_xml_set_value_from_string(pspec, info->properties[i].value,
+	/* this should catch all properties wanting a GtkWidget
+         * subclass.  We also look for types that could hold a
+         * GtkWidget in order to catch things like the
+         * GtkAccelLabel::accel_object property.  Since we don't do
+         * any handling of GObject or GtkObject directly in
+         * glade_xml_set_value_from_string, this shouldn't be a
+         * problem. */
+	if (g_type_is_a(GTK_TYPE_WIDGET, G_PARAM_SPEC_VALUE_TYPE(pspec)) ||
+	    g_type_is_a(G_PARAM_SPEC_VALUE_TYPE(pspec), GTK_TYPE_WIDGET)) {
+	    deferred_props = g_list_prepend(deferred_props,
+					    &info->properties[i]);
+	    continue;
+	}
+
+	if (glade_xml_set_value_from_string(xml, pspec,
+					    info->properties[i].value,
 					    &param.value)) {
 	    param.name = info->properties[i].name;
 	    g_array_append_val(props_array, param);
@@ -1278,6 +1271,15 @@ glade_standard_build_widget(GladeXML *xml, GType widget_type,
 	g_array_index(props_array, GParameter, i).name = NULL;
 	g_value_unset(&g_array_index(props_array, GParameter, i).value);
     }
+
+    /* handle deferred properties */
+    for (tmp = deferred_props; tmp; tmp = tmp->next) {
+	GladeProperty *prop = tmp->data;
+
+	glade_xml_handle_widget_prop(xml, widget, prop->name, prop->value);
+    }
+    g_list_free(tmp);
+
     g_array_set_size(props_array, 0);
     g_type_class_unref(oclass);
 
@@ -1334,7 +1336,7 @@ glade_standard_build_children(GladeXML *self, GtkWidget *parent,
 		continue;
 	    }
 
-	    if (glade_xml_set_value_from_string(pspec,
+	    if (glade_xml_set_value_from_string(self, pspec,
 			info->children[i].properties[j].value, &value)) {
 		gtk_container_child_set_property(GTK_CONTAINER(parent), child,
 						 name, &value);
@@ -1507,12 +1509,12 @@ glade_xml_widget_destroy(GtkObject *object, GladeXML *xml)
     g_return_if_fail(GTK_IS_OBJECT(object));
     g_return_if_fail(GLADE_IS_XML(xml));
 
-    name = g_object_get_data(G_OBJECT(object), glade_xml_name_tag);
+    name = g_object_get_qdata(G_OBJECT(object), glade_xml_name_id);
 
     if (!name) return;
     g_hash_table_remove(xml->priv->name_hash, name);
-    g_object_set_data(G_OBJECT(object), glade_xml_tag, NULL);
-    g_object_set_data(G_OBJECT(object), glade_xml_name_tag, NULL);
+    g_object_set_qdata(G_OBJECT(object), glade_xml_tree_id, NULL);
+    g_object_set_qdata(G_OBJECT(object), glade_xml_name_id, NULL);
 }
 
 /**
@@ -1552,8 +1554,8 @@ glade_xml_set_common_params(GladeXML *self, GtkWidget *widget,
 #endif
 
     /* store this information as data of the widget. */
-    gtk_object_set_data(GTK_OBJECT(widget), glade_xml_tag, self);
-    gtk_object_set_data(GTK_OBJECT(widget), glade_xml_name_tag, info->name);
+    g_object_set_qdata(G_OBJECT(widget), glade_xml_tree_id, self);
+    g_object_set_qdata(G_OBJECT(widget), glade_xml_name_id, info->name);
     /* store widgets in hash table, for easy lookup */
     g_hash_table_insert(self->priv->name_hash, info->name, widget);
 
@@ -1565,6 +1567,25 @@ glade_xml_set_common_params(GladeXML *self, GtkWidget *widget,
 			    G_CALLBACK(glade_xml_widget_destroy),
 			    G_OBJECT(self), 0);
 
+    /* handle any deferred properties using this widget */
+    tmp = self->priv->deferred_props;
+    while (tmp) {
+	GladeDeferredProperty *dprop = tmp->data;
+
+	if (!strcmp(info->name, dprop->prop_value)) {
+	    tmp = tmp->next;
+	    self->priv->deferred_props =
+		g_list_remove(self->priv->deferred_props, dprop);
+
+	    g_object_set(G_OBJECT(dprop->target),
+			 dprop->prop_name, G_OBJECT(widget),
+			 NULL);
+	    g_free(dprop);
+	} else {
+	    tmp = tmp->next;
+	}
+    }
+
     if (data && data->build_children && info->children)
 	data->build_children(self, widget, info);
 }
